# -*- coding: utf-8 -*-
# Author: Manuel de la Pena <manuel@canonical.com>
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
"""Test the interaction of the view with the auto generated code."""

from unittest import TestCase
from mocker import MATCH, Mocker
from PyQt4.QtCore import Qt

from ubuntu_sso.qt.gui import (
    ChooseSignInPage,
    CurrentUserSignInPage,
    EmailVerificationPage,
    ErrorPage,
    SetupAccountPage,
    SuccessPage,
    TosPage)
# in linux we will be having a pylint warning because the code is not
# compiled since we are just targeting windows with it, later
# in the iteration we can work on using it on kde
# pylint: disable=F0401,E0611
from ubuntu_sso.qt.choose_sign_in_ui import Ui_ChooseSignInPage
from ubuntu_sso.qt.current_user_sign_in_ui import Ui_CurrentUserSignInPage
from ubuntu_sso.qt.email_verification_ui import Ui_EmailVerificationPage
from ubuntu_sso.qt.error_message_ui import Ui_ErrorPage
from ubuntu_sso.qt.terms_and_conditions_ui import Ui_TosPage
from ubuntu_sso.qt.setup_account_ui import Ui_SetUpAccountPage
from ubuntu_sso.qt.success_message_ui import Ui_SuccessPage
# pylint: enable=F0401


class FakeController(object):
    """A fake controller for the tests."""

    # pylint: disable=C0103
    def setupUi(self, view):
        """Fake the setup."""
    # pylint: enable=C0103


class ChooseSignInPageTestCase(TestCase):
    """Test the ChooseSignInPage."""

    def setUp(self):
        """Set up the tests."""
        super(ChooseSignInPageTestCase, self).setUp()
        self.ui = Ui_ChooseSignInPage()
        self.controller = FakeController()
        self.widget = ChooseSignInPage(self.ui, self.controller)

    def tearDown(self):
        self.widget.close()

    def test_constructor(self):
        """Test the constructor of the class."""
        # used a mocked ui for this
        mocker = Mocker()
        ui = mocker.mock()
        controller = mocker.mock()
        ui.setupUi(MATCH(lambda x: isinstance(x, ChooseSignInPage)))
        controller.setupUi(MATCH(lambda x: isinstance(x, ChooseSignInPage)))
        mocker.replay()
        self.widget = ChooseSignInPage(ui, controller)

    def test_next_id(self):
        """Test the nextId method."""
        next_id = 1
        self.widget.next = next_id
        self.assertEqual(next_id, self.widget.nextId())

    def test_existing_account_button(self):
        """Test the property."""
        self.assertEqual(self.ui.existing_account_button,
                         self.widget.existing_account_button)

    def test_new_account_button(self):
        """Test the property."""
        self.assertEqual(self.ui.setup_account_button,
                         self.widget.new_account_button)


class CurrentUserSignInPageTestCase(TestCase):
    """Test the CurrentUserSignInPage."""

    def setUp(self):
        """Set up the tests."""
        super(CurrentUserSignInPageTestCase, self).setUp()
        self.ui = Ui_CurrentUserSignInPage()
        self.controller = FakeController()
        self.widget = CurrentUserSignInPage(self.ui, self.controller)

    def test_constructor(self):
        """Test the constructor of the class."""
        mocker = Mocker()
        ui = mocker.mock()
        controller = mocker.mock()
        ui.setupUi(MATCH(lambda x: isinstance(x, CurrentUserSignInPage)))
        controller.setupUi(MATCH(lambda x: isinstance(x,
                                                      CurrentUserSignInPage)))
        mocker.replay()
        self.widget = ChooseSignInPage(self.ui, self.controller)

    def test_email(self):
        """Test the email property."""
        # set the email in the ui and test that the result is the expected one
        email = 'manuel@canonical.com'
        self.ui.email_edit.setText(email)
        self.assertEqual(email, self.widget.email)

    def test_email_edit(self):
        """Test the email edit property."""
        self.assertEqual(self.ui.email_edit, self.widget.email_edit)

    def test_password(self):
        """Test the password property."""
        # set password in the ui and test that the result is the correct one
        password = 'password'
        self.ui.password_edit.setText(password)
        self.assertEqual(password, self.widget.password)

    def test_password_edit(self):
        """Test the password edit property."""
        self.assertEqual(self.ui.password_edit, self.widget.password_edit)

    def test_forgot_password_label(self):
        """Test the password forgto label."""
        self.assertEqual(self.ui.forgot_label,
                         self.widget.forgot_password_label)

    def test_signin_button(self):
        """Test the sign in button property."""
        self.assertEqual(self.ui.sign_in_button, self.widget.sign_in_button)


class EmailVerificationPageTestCase(TestCase):
    """Test the EmailVerificationPage."""

    def setUp(self):
        """Set up the tests."""
        super(EmailVerificationPageTestCase, self).setUp()
        self.ui = Ui_EmailVerificationPage()
        self.controller = FakeController()
        self.widget = EmailVerificationPage(self.ui, self.controller)

    def test_constructor(self):
        """Test the constructor of the class."""
        mocker = Mocker()
        ui = mocker.mock()
        controller = mocker.mock()
        ui.setupUi(MATCH(lambda x: isinstance(x, EmailVerificationPage)))
        controller.setupUi(MATCH(lambda x: isinstance(x,
                                                      EmailVerificationPage)))
        mocker.replay()
        self.widget = EmailVerificationPage(self.ui, self.controller)

    def test_verification_code(self):
        """Test the verification code property."""
        verification = 'weird thing'
        self.ui.verification_code_edit.setText(verification)
        self.assertEqual(verification, self.widget.verification_code)

    def test_verification_code_edit(self):
        """Test the vericiation code edit property."""
        self.assertEqual(self.ui.verification_code_edit,
                         self.widget.verification_code_edit)

    def test_next_button(self):
        """Test the next button property."""
        self.assertEqual(self.ui.next_button, self.widget.next_button)


class TosPageTestCase(TestCase):
    """Test the TosPage."""

    def setUp(self):
        """Set up the tests."""
        super(TosPageTestCase, self).setUp()
        self.ui = Ui_TosPage()
        self.controller = FakeController()
        self.widget = TosPage(self.ui, self.controller)

    def test_constructor(self):
        """Test the constructor of the class."""
        mocker = Mocker()
        ui = mocker.mock()
        controller = mocker.mock()
        ui.setupUi(MATCH(lambda x: isinstance(x, TosPage)))
        controller.setupUi(MATCH(lambda x: isinstance(x, TosPage)))
        mocker.replay()
        self.widget = TosPage(self.ui, self.controller)

    def test_next_id(self):
        """Test the nextId method."""
        self.assertEqual(-1, self.widget.nextId())

    def test_webkit(self):
        """Test the webkit property."""
        self.assertEqual(self.ui.terms_webkit, self.widget.webkit)


class SetUpAccountPageTestCase(TestCase):
    """Test the SetUpAccountPage."""

    def setUp(self):
        """Set up the tests."""
        super(SetUpAccountPageTestCase, self).setUp()
        self.ui = Ui_SetUpAccountPage()
        self.controller = FakeController()
        self.widget = SetupAccountPage(self.ui, self.controller)

    def test_constructor(self):
        """Test the constrcutor of the class."""
        mocker = Mocker()
        ui = mocker.mock()
        controller = mocker.mock()
        ui.setupUi(MATCH(lambda x: isinstance(x, SetupAccountPage)))
        controller.setupUi(MATCH(lambda x: isinstance(x, SetupAccountPage)))
        mocker.replay()
        self.widget = SetupAccountPage(self.ui, self.controller)

    def test_next_id(self):
        """Test the newxtId method."""
        next_id = 1
        self.widget.next = next_id
        self.assertEqual(next_id, self.widget.nextId())

    def test_set_line_edit_validation_rule(self):
        """Test the method that sets the validation runles."""

    def test_name(self):
        """Test the property that returns the name."""
        name = 'Manuel'
        self.ui.first_name_edit.setText(name)
        self.assertEqual(name, self.widget.name)

    def test_name_edit(self):
        """Test the property that returns the name edit."""
        self.assertEqual(self.ui.first_name_edit, self.widget.name_edit)

    def test_last_name(self):
        """Test the property that returns the last name."""
        last_name = 'de la Pena'
        self.ui.last_name_edit.setText(last_name)
        self.assertEqual(last_name, self.widget.last_name)

    def test_last_name_edit(self):
        """Test the property that returns the last name edit."""
        self.assertEqual(self.ui.last_name_edit, self.widget.last_name_edit)

    def test_email(self):
        """Test the property that returns the email."""
        email = 'manuel@canonical.com'
        self.ui.email_edit.setText(email)
        self.assertEqual(email, self.widget.email)

    def test_email_edit(self):
        """Test the property that returns the email edit."""
        self.assertEqual(self.ui.email_edit, self.widget.email_edit)

    def test_retyped_email(self):
        """Test the property that returns the re-typed email."""
        retyped_email = 'manuel@canonical.com'
        self.ui.confirm_email_edit.setText(retyped_email)
        self.assertEqual(retyped_email, self.widget.retyped_email)

    def test_confirm_email_edit(self):
        """Test the property that returns the confirm edit."""
        self.assertEqual(self.ui.confirm_email_edit,
                         self.widget.confirm_email_edit)

    def test_password_info_label(self):
        """Test the property that returns the password info label."""
        self.assertEqual(self.ui.password_info_label,
            self.widget.password_info_label)

    def test_pasword(self):
        """Test the property that returns the password."""
        password = 'password'
        self.ui.password_edit.setText(password)
        self.assertEqual(password, self.widget.password)

    def test_password_edit(self):
        """Test the property that returns the password edit."""
        self.assertEqual(self.ui.password_edit, self.widget.password_edit)

    def test_retyped_password(self):
        """Test the property that returns the retyped password."""
        password = 'password'
        self.ui.confirm_password_edit.setText(password)
        self.assertEqual(password, self.widget.retyped_password)

    def test_confirm_password_edit(self):
        """Test the property that returns the password confirm edit."""
        self.assertEqual(self.ui.confirm_password_edit,
                         self.widget.confirm_password_edit)

    def test_captcha_refresh_label(self):
        """Test the property that returns the captcha refresh label."""
        self.assertEqual(self.ui.refresh_label,
                         self.widget.captcha_refresh_label)

    def test_captcha_solution(self):
        """Test the property that returns the capthca solution."""
        solution = 'solution'
        self.ui.captcha_solution_edit.setText(solution)
        self.assertEqual(solution, self.widget.captcha_solution)

    def test_tos_agreed_false(self):
        """Test the property that returns if the tos was agreed."""
        self.ui.terms_checkbox.setCheckState(Qt.Unchecked)
        self.assertFalse(self.widget.terms_and_conditions_agreed)

    def test_tos_agreed_true(self):
        """Test the property that returns if the tos was agreed."""
        # check the box and test that the result is the expected one
        self.ui.terms_checkbox.setCheckState(Qt.Checked)
        self.assertTrue(self.widget.terms_and_conditions_agreed)

    def test_tos_button(self):
        """Test the property that returns the tos button."""
        self.assertEqual(self.ui.terms_button,
                         self.widget.terms_and_conditions_button)

    def test_set_up_button(self):
        """Test the property that returns the set up button."""
        self.assertEqual(self.ui.set_up_button, self.widget.set_up_button)


class SuccessPageTestCase(TestCase):
    """Test that the correct widgets are used."""

    def setUp(self):
        """Setup tests."""
        super(SuccessPageTestCase, self).setUp()
        self.ui = Ui_SuccessPage()
        self.controller = FakeController()
        self.widget = SuccessPage(self.ui, self.controller)

    def test_constructor(self):
        """Test that the constructor works as expected."""
        mocker = Mocker()
        ui = mocker.mock()
        controller = mocker.mock()
        ui.setupUi(MATCH(lambda x: isinstance(x, SuccessPage)))
        controller.setupUi(MATCH(lambda x: isinstance(x, SuccessPage)))
        mocker.replay()
        self.widget = SuccessPage(self.ui, self.controller)

    def test_success_message_label(self):
        """Test that the correct label is returned."""
        self.assertEqual(self.ui.success_message_label,
                         self.widget.success_message_label)


class ErrorPageTestCase(TestCase):
    """Test that the correct widgets are used."""

    def setUp(self):
        """Setup tests."""
        super(ErrorPageTestCase, self).setUp()
        self.ui = Ui_ErrorPage()
        self.controller = FakeController()
        self.widget = ErrorPage(self.ui, self.controller)

    def test_constructor(self):
        """Test that the constructor works as expected."""
        mocker = Mocker()
        ui = mocker.mock()
        controller = mocker.mock()
        ui.setupUi(MATCH(lambda x: isinstance(x, ErrorPage)))
        controller.setupUi(MATCH(lambda x: isinstance(x, ErrorPage)))
        mocker.replay()
        self.widget = ErrorPage(self.ui, self.controller)

    def test_success_message_label(self):
        """Test that the correct label is returned."""
        self.assertEqual(self.ui.error_message_label,
                         self.widget.error_message_label)
